//////////////////////////////////////////////////////////////////////////////////////
//  
//  FMPluginSolidColor.cpp - Sample external function for FileMaker plug-in
//
//  Copyright 2003-2004 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky & HOnza Koudelka
//
////////////////////////////////////////////////////////////////////////////////
//
//  This example is based on 24U FM Template, a stationery for building
//  FileMaker plug-ins from 24u Software.
//
//  The latest version of 24U FM Template is available for download from:
//
//  http://24uSoftware.com/FMTemplate  or  http://www.NewMillennium.com/
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source with references to
//  24U Software, we require that you make it clear in the source that
//  the code was descended from 24U Sample Code, but that you've made changes.
//
////////////////////////////////////////////////////////////////////////

//
//  This function takes three numeric parameters - the red, green and blue
//  components of an RGB color. It returns a one-pixel picture of the color
//  specified as a result. The result (of type container) can be enlarged
//  to fill an arbitrary rectangle with specified color.
//
////////////////////////////////////////////////////////////////////////////////


#include "FMPlugin.h"
#include "FMTemplate/FMTemplate.h"

static FMX_Hdl CreateSinglePixelGIF( unsigned char R, unsigned char G, unsigned char B );

#pragma mark Plugin_SolidColor
FMX_PROC(fmx::errcode) Plugin_SolidColor(short funcId, const fmx::ExprEnv& environment, const fmx::DataVect& dataVect, fmx::Data& result)
{
    #pragma unused(funcId,environment)                                          // This function is supposed to work anywhere
    
    fmx::errcode errorResult = -1;                                              // Prepare an error for the case we fail
    
    if( dataVect.Size() == 3 )                                                  // Require exactly 3 parameters (red, green, blue)
    {
        const fmx::FixPt&       redComponent   = dataVect.AtAsNumber(0);        // Extract the parameters as integer
        const fmx::FixPt&       greenComponent = dataVect.AtAsNumber(1);        //  values of the required color's
        const fmx::FixPt&       blueComponent  = dataVect.AtAsNumber(2);        //  red, green and blue components
        fmx::BinaryDataAutoPtr  picture;                                        // Prepare a picture stream
        short                   size[2] = {0x0001,0x0001};                      //  of the size 1 x 1 pixel
        fmx::QuadCharAutoPtr    sizeTypeID('S','I','Z','E');
        fmx::QuadCharAutoPtr    gifTypeID('G','I','F','f');                     //  of the type "GIFf"
        FMX_Hdl                 gif = NULL;                                     //  with data to be constructed later
        
        gif = CreateSinglePixelGIF( redComponent.AsLong(),                      // Get the data for the stream as GIF
                                    greenComponent.AsLong(),
                                    blueComponent.AsLong() );

        if( gif )                                                               // Were we successfull getting the GIF?
        {
            FMX_LockHandle(gif);                                                // Lock the GIF's memory so that we can manipulate it

                picture -> Add( *gifTypeID, (fmx::ulong)FMX_GetHandleSize(gif), *gif );     // Add the GIF to the picture stream
                picture -> Add( *sizeTypeID, 4, &size );                                    // Add the size info to the stream
                
                result.SetBinaryData( *picture );                               // Set the function result to our picture stream

            FMX_UnlockHandle(gif);                                              // Unlock the GIF's memory and dispose of it,
            FMX_DisposeHandle( gif );                                           //  as we don't need it any more

            errorResult = 0;                                                    // We have successfully set the result => no error
        }
        else
        {
            errorResult = FMX_MemoryError();                                    // GIF could not be created => find out why
        }
    }

    return(errorResult);
}

static FMX_Hdl CreateSinglePixelGIF( unsigned char R, unsigned char G, unsigned char B )
{
    unsigned char    onePixelGif[] = { 0x47, 0x49, 0x46, 0x38, 0x37, 0x61,  //  header (GIF87a)
                                       0x01, 0x00,                          //  width in little endian 
                                       0x01, 0x00,                          //  height in little endian
                                       0x80,                                //  flags 
                                       0x00,                                //  back color index
                                       0x00,                                //  filler
                                       0xFF, 0x00, 0x00,                    //  red (RR GG BB)
                                       0xFF, 0xFF, 0xFF,                    //  white (RR GG BB)
                                       0x2C,                                //  begin marker
                                       0x00, 0x00,                          //  left in little endian
                                       0x00, 0x00,                          //  top in little endian
                                       0x01, 0x00,                          //  width in little endian
                                       0x01, 0x00,                          //  height in little endian
                                       0x00,                                //  filler
                                       0x02, 0x02, 0x44, 0x01,              //  LZW
                                       0x00,                                //  LZW end mark
                                       0x3B                                 //  end marker
                                     };
    
    FMX_Hdl result = FMX_NewHandle( sizeof(onePixelGif) );                  //  allocate memory for one-pixel GIF
    
    if( result )
    {
        memcpy( *result, onePixelGif, sizeof(onePixelGif) );                //  copy default one-pixel GIF to the buffer
        
        (*result)[13] = (char)R;                                            //  modify the color of the one-pixel
        (*result)[14] = (char)G;
        (*result)[15] = (char)B;
    }
    
    return result;
}

